﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace rbcom {
    class AliasCommand : rbcom.Command {

        protected bool UnsetMode { get; set; }
        protected bool ResetMode { get; set; }

        public AliasCommand() : base() {
            this.UnsetMode = false;
            this.ResetMode = false;

            this.Options.Add("unset", "指定したエイリアスの削除をします", delegate(string v) { this.UnsetMode = true; });
            this.Options.Add("reset", "エイリアスのリセット (全削除) を行います", delegate(string v) { this.ResetMode = true; });

            this.Usage = String.Format("USAGE: {0} [options] RUBY_VERSION [ALIAS [, ALIAS...]]", this.Name);
            this.Description = @"Rubyのバージョンに対するエイリアス設定を行います
エイリアスはバージョン指定の代わりになるものです

RUBY_VERSIONの形式は [{ruby}-]{x}[.{y}[.{z}]][-p{ddd}][-{platform}] です.
{}で囲われた部分は可変であり、[]で囲われた部分は省略可能部分です.
それぞれの意味は次のとおりです

  {ruby}        Ruby名 'ruby' だったり 'jruby' だったりします
  {x}.{y}.{z}   Rubyバージョン番号
  {ddd}         パッチレベル
  {platform}    Rubyの実装プラットフォーム (mswin32等)
";
        }

        protected override bool Invoke(string[] args) {
            rbcom.RubyProgram ruby = this.Rubies.Find(args[0]).First();
            if (ruby == null) {
                System.Console.Error.WriteLine("対象となるRubyが見つかりませんでした: {0]", args[0]);
                return false;
            }
#if DEBUG
            System.Console.Error.WriteLine("[INFO] Target Ruby is {0}", ruby.ToString());
#endif
            if (ruby.Aliases == null) {
                ruby.Aliases = new HashSet<string>();
            }
            if (this.ResetMode) {
                ruby.Aliases.Clear();
            } else {
                string[] aliases = args.Skip(1).ToArray();

                if (this.UnsetMode) {
                    foreach (string alias in aliases) {
                        System.Console.WriteLine("Unset alias '{0}' from {1}", alias, ruby.ToString());

                        ruby.Aliases.Remove(alias);
                    }
                } else {
                    foreach (string alias in aliases) {
                        System.Console.WriteLine("Set alias '{0}' to {1}", alias, ruby.ToString());
                        ruby.Aliases.Add(alias);
                    }
                }
            }

            return true;
        }

        protected override bool Validate(string[] args) {
            return (args.Length > 0);
        }

        static void Main(string[] args) {
            Run(new AliasCommand(), args);
        }
    }
}
