﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace rbcom {
    class DelCommand : rbcom.Command {

        public DelCommand() : base() {
            this.Usage = String.Format("USAGE: {0} [options] RUBY_VERSION [RUBY_VERSION...]", this.Name);
            this.Description = @"指定したRubyを管理からか除外します";
        }

        protected override bool Invoke(string[] args) {
            foreach (String arg in args) {
                // 対象を検索
                List<RubyProgram> rubies = this.Rubies.Find(arg, true);

                // 消したものリスト
                List<RubyProgram> removed = null;

                if (rubies.Count == 0) {
                    // 見つからなかった
                    System.Console.WriteLine("バージョン '{0}' に一致するRubyは見つかりませんでした", arg);
                } else if (rubies.Count == 1) {
                    // 一件見つかったので外す
                    removed = this.Rubies.Delete(rubies[0]);
                } else {
                    removed = new List<RubyProgram>();

                    // 複数見つかったので、尋ねる
                    System.Console.WriteLine("対象となるRubyが {0} 件見つかりました", rubies.Count);
                    System.Console.WriteLine();
                    for (int i = 0; i < rubies.Count; i++) {
                        RubyProgram ruby = rubies[i];
                        System.Console.WriteLine("  {0}) {1} を消します", i, ruby);
                    }
                    System.Console.WriteLine("  all) すべて消します");
                    System.Console.WriteLine();
                    System.Console.Write("どれを消しますか？> ");
                    string target = System.Console.ReadLine();
                    target = target.Trim();

                    if (target.Length > 0) {
                        if (target.ToLower() == @"all") {
                            // 全件削除
                            foreach (RubyProgram ruby in rubies) {
                                foreach (RubyProgram rm in this.Rubies.Delete(ruby)) {
                                    removed.Add(rm);
                                }
                            }
                        } else {
                            int? num = null;
                            try {
                                num = Int32.Parse(target);
                            } catch (FormatException e) {
                                System.Console.WriteLine(e.Message);
                                // 不正な入力なら無視
                                num = null;
                            }

                            if (num.HasValue && num >= 0 && num < rubies.Count) {
                                foreach (RubyProgram rm in this.Rubies.Delete(rubies[num.Value])) {
                                    removed.Add(rm);
                                }
                            }
                        }
                    } else {
                        // 未入力はなにもしない
                    }
                }

                // 削除済みを表示
                foreach (RubyProgram rm in removed) {
                    System.Console.WriteLine("管理下から削除されました:{0}", rm);
                }
            }

            return true;
        }

        protected override bool Validate(string[] args) {
            return (args.Length >= 1);
        }

        static void Main(string[] args) {
            Run(new DelCommand(), args);
        }
    }
}
