﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;

using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;

namespace rbcom {

    [DataContractAttribute]
    public class RubyProgram {

        [DataMemberAttribute(Name = "command", Order = 0, IsRequired = true, EmitDefaultValue = false)]
        public string Command { get; set; }

        [DataMemberAttribute(Name = "location", Order = 1, IsRequired = true, EmitDefaultValue = false)]
        public string Location { get; set; }

        [DataMemberAttribute(Name = "name", Order = 2, IsRequired = true, EmitDefaultValue = false)]
        public string Name { get; set; }

        [DataMemberAttribute(Name = "version", Order = 3, IsRequired = true, EmitDefaultValue = false)]
        public RubyVersion Version { get; set; }

        [DataMemberAttribute(Name = "arch", Order = 5, IsRequired = false, EmitDefaultValue = false)]
        public string Architecture { get; set; }

        [DataMemberAttribute(Name = "platform", Order = 6, IsRequired = false, EmitDefaultValue = false)]
        public string Platform { get; set; }

        [DataMemberAttribute(Name = "aliases", Order = 7, IsRequired = false, EmitDefaultValue = false)]
        public HashSet<string> Aliases { get; set; }

        public RubyProgram() {
            this.Aliases = new HashSet<string>();
        }

        public override bool Equals(object obj) {
            if (obj.GetType().IsSubclassOf(this.GetType())) {
                return this.Equals((RubyProgram)obj);
            }
            return base.Equals(obj);
        }

        public bool Equals(RubyProgram obj) {
            // コマンドの比較
            if (this.Command != obj.Command) {
                Debug.WriteLine("[DEBUG] Unmatched: {0} <=> {1}", this.Command, obj.Command);
                return false;
            }

            // 場所の比較
            if (this.Location != obj.Location) {
                Debug.WriteLine("[DEBUG] Unmatched: {0} <=> {1}", this.Location, obj.Location);
                return false;
            }

            // 名称の比較
            if (this.Name != obj.Name) {
                Debug.WriteLine("[DEBUG] Unmatched: {0} <=> {1}", this.Name, obj.Name);
                return false;
            }

            // バージョン
            if (this.Version == null && obj.Version != null) {
                Debug.WriteLine("[DEBUG] Unmatched: {0} <=> {1}", null, obj.Version);
                return false;
            } else if (this.Version != null && obj.Version == null) {
                Debug.WriteLine("[DEBUG] Unmatched: {0} <=> {1}", this.Version, null);
                return false;
            } else if (this.Version != null && obj.Version != null && !this.Version.Equals(obj.Version)) {
                Debug.WriteLine("[DEBUG] Unmatched: {0} <=> {1}", this.Version, obj.Version);
                return false;
            }

            // アーキテクチャ
            if (this.Architecture != obj.Architecture) {
                Debug.WriteLine("[DEBUG] Unmatched: {0} <=> {1}", this.Architecture, obj.Architecture);
                return false;
            }

            // プラットフォーム
            if (this.Platform != obj.Platform) {
                Debug.WriteLine("[DEBUG] Unmatched: {0} <=> {1}", this.Platform, obj.Platform);
                return false;
            }

            return true;
        }

        public override int GetHashCode() {
            int cmdHash = 0;
            int nameHash = 0;
            int locationHash = 0;
            int versionHash = 0;
            int patchHash = 0;
            int archHash = 0;
            int platformHash = 0;

            if (this.Command != null) {
                cmdHash = this.Command.GetHashCode();
            }
            if (this.Name != null) {
                nameHash = this.Name.GetHashCode();
            }
            if (this.Location != null) {
                locationHash = this.Location.GetHashCode();
            }
            if (this.Version != null) {
                versionHash = this.Version.GetHashCode();
            }
            if (this.Platform != null) {
                platformHash = this.Platform.GetHashCode();
            }

            int hashCode = cmdHash ^ locationHash ^ nameHash ^ versionHash ^ patchHash ^ archHash ^ platformHash;

            return hashCode;
        }

        public override string ToString() {
            if (this.Architecture != null && this.Architecture.Length > 0) {
                return String.Format("{0} {1} [{2}-{3}]", this.Name, this.Version, this.Architecture, this.Platform);
            } else {
                return String.Format("{0} {1} [{2}]", this.Name, this.Version, this.Platform);
            }
        }
    }
}
